"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CallParametersDataParameter = exports.CallParameters = void 0;
const OracleDBHelper_1 = require("./OracleDBHelper");
class CallParameters {
    constructor() {
        this.isFunction = true;
        this.data = [];
    }
    prepareFromRequestBody(body) {
        this.data = !this.data ? [] : this.data;
        for (const key in body) {
            let value = body[key];
            if (value instanceof Object && !(value instanceof Date)) {
                // tslint:disable-next-line: no-null-keyword
                value = null;
            }
            const dbFieldName = 'p_' + key.toLowerCase();
            this.data.push(new CallParametersDataParameter(dbFieldName, value));
        }
    }
    prepareAndReturnRequestBody(body) {
        this.prepareFromRequestBody(body);
        return this.data;
    }
}
exports.CallParameters = CallParameters;
class CallParametersDataParameter {
    constructor(name, value, isComplexType = false, type) {
        this.isComplexType = false;
        this.name = name;
        this.isComplexType = isComplexType;
        this.type = type;
        this.value = value;
    }
    get value() {
        return this._value;
    }
    set value(value) {
        if (this.isComplexType) {
            this.setComplexValue(value);
        }
        else {
            this.setSimpleValue(value);
        }
    }
    setComplexValue(value) {
        if (!(value instanceof Array)) {
            throw new Error('Wrong type passed for complex type parameter. Should be array.');
        }
        if (!this.type) {
            throw new Error('Cannot set complex type value. Type is not defined.');
        }
        this._value = value;
    }
    setSimpleValue(value) {
        if (value instanceof Object && !(value instanceof Date)) {
            throw new Error('Wrong type passed for simple type parameter');
        }
        this._value = value;
    }
    get type() {
        return this._type;
    }
    set type(type) {
        this._type = type;
    }
    get objectClass() {
        return this._objectClass;
    }
    set objectClass(objectClass) {
        this._objectClass = objectClass;
    }
    get bindParameterValue() {
        if (this.value === undefined) {
            throw new Error(`Cannot return bind parameter. Value for "${this.name}" is not defined.`);
        }
        if (this.isComplexType) {
            return this.prepareComplexTypeBind();
        }
        return this._value;
    }
    prepareComplexTypeBind() {
        if (this.objectClass === undefined) {
            throw new Error(`Cannot return bind parameter. DB Object Class "${this.type}" is not defined.`);
        }
        this.dirtyTmpUDFFix();
        return new this._objectClass(this.value);
    }
    // TODO remove
    dirtyTmpUDFFix() {
        const updatedValue = this.value;
        if (!(updatedValue instanceof Array)) {
            return;
        }
        for (const singleValue of updatedValue) {
            Object.keys(singleValue).map((key) => {
                if (singleValue[key] !== '') {
                    return;
                }
                if (RegExp(/[A-Z]{3}_UDF_.+/i).test(key)) {
                    singleValue[key] = OracleDBHelper_1.OracleDBHelper.NULL;
                }
            });
        }
        this.value = updatedValue;
    }
}
exports.CallParametersDataParameter = CallParametersDataParameter;
//# sourceMappingURL=CallParameters.js.map